from threading import Event


class Message:
    def __init__(self, timeout=10):
        self._ready = Event()
        self._timeout = timeout
        self._response = None

    @property
    def result(self):
        received = self._ready.wait(timeout=self._timeout)
        if not received:
            raise MqttError("CONNECTION", "No Response Received")
        if not self._response['ok']:
            raise MqttError(self._response['errorCode'], self._response['error'])
        return self._response['data']

    @result.setter
    def result(self, dato):
        self._response = dato
        self._ready.set()

    def __len__(self):
        return len(self.result)

    def __getitem__(self, key):
        return self.result[key]

    def __iter__(self):
        return self.result.__iter__()

    def __contains__(self, key):
        return key in self.result


class MqttError(Exception):
    def __init__(self, error_code, description):
        self.error_code = error_code
        self.description = description



import inspect
import logging
import logging.handlers
import os
import platform
import sys
import traceback
import onesaitplatform.common.config as config

try:
    default_dir = ".onesaitplatform"
    if config.LOG_FOLDER:
        d = os.path.join(config.LOG_FOLDER, default_dir)
    else:
        d = os.path.join(os.path.expanduser("~"), default_dir)

    try:
        if not os.path.exists(d):
            os.mkdir(d)
    except:
        d = default_dir
        if not os.path.exists(d):
            os.mkdir(d)

finally:
    log_file_path = os.path.join(d, "onesaitplatform-python-client.log")


class log(object):

    handler = None
    plugin_id = "onesaitplatform-python-client"

    @staticmethod
    def error(text):
        logger = logging.getLogger(log.plugin_id)
        logger.error(text)

    @staticmethod
    def info(text):
        logger = logging.getLogger(log.plugin_id)
        logger.info(text)

    @staticmethod
    def warning(text):
        logger = logging.getLogger(log.plugin_id)
        logger.warning(text)

    @staticmethod
    def debug(text):
        logger = logging.getLogger(log.plugin_id)
        logger.debug(text)

    @staticmethod
    def set_level(level=logging.DEBUG):
        logger = logging.getLogger(log.plugin_id)
        logger.setLevel(level)

    @staticmethod
    def last_exception(msg):
        exc_type, exc_value, exc_traceback = sys.exc_info()
        log.error(
            msg + '\n  '.join(traceback.format_exception(exc_type,
                              exc_value, exc_traceback)))

    @staticmethod
    def init_logging():
        try:
            """ set up rotating log file handler with custom formatting """
            log.handler = logging.handlers.RotatingFileHandler(
                log_file_path, maxBytes=1024 * 10, backupCount=5)
            formatter = logging.Formatter(
                "%(asctime)s %(levelname)-8s %(message)s")
            log.handler.setFormatter(formatter)
            logger = logging.getLogger(log.plugin_id)  # root logger
            logger.setLevel(logging.INFO)
            logger.addHandler(log.handler)
            log.info("----------------- Start Log -----------------")
        except Exception:
            pass

    @staticmethod
    def remove_logging():
        logger = logging.getLogger(log.plugin_id)
        logger.removeHandler(log.handler)
        del log.handler

    @staticmethod
    def log_Stack_trace():
        logger = logging.getLogger(log.plugin_id)
        logger.debug("log_Stack_trace")
        for x in inspect.stack():
            logger.debug(x)



import os
import sys

# -------- Configuration --------
# Application
LOG_FOLDER = None

# Client
USER_AGENT = "onesaitplatform:PythonClient"
HOST = "www.onesaitplatform.online"
DEBUG_TRACE_LIMIT = 25
PROTOCOL = "https"

# DigitalClient
IOT_BROKER_HOST = "iotbrokerservice"
IOT_BROKER_AVOID_SSL_CERTIFICATE = True
IOT_BROKER_PORT = 19000
IOT_BROKER_PATH = "/iot-broker"
IOT_CLIENT = "Client4Notebook"
IOT_CLIENT_TOKEN = "674a6e05348a468787e5af4acdf5b3df"
BATCH_QUERY_SIZE = 2000

# ApiManagerClient
API_MANAGER_HOST = "apimanagerservice"
API_MANAGER_PORT = 19100
API_MANAGER_AVOID_SSL_CERTIFICATE = True
API_MANAGER_PATH = "/api-manager/services/management"
API_CALLER_PATH = "/api-manager/server/api"
API_USER_TOKEN = "b32522cd73e84ddda519f1dff9627f40"

# FileManager
FILE_MANAGER_HOST = "controlpanelservice"
FILE_MANAGER_BINARY_FILES_PATH = "/controlpanel/binary-repository"
FILE_MANAGER_FILES_PATH = "/controlpanel/files"
FILE_MANAGER_PATH = "/controlpanel"
USER_TOKEN = "Bearer ..."

# ModelServiceClient



import requests
from string import Template
import json
import logging
from onesaitplatform.base import Client
import onesaitplatform.common.config as config
from onesaitplatform.enum import QueryType, RestMethods, RestHeaders
from onesaitplatform.common.log import log

try:
    logging.basicConfig()
    log = logging.getLogger(__name__)
except:
    log.init_logging()


class ApiManagerClient(Client):
    """
    Class ApiManagerClient to connect with Api-Manager and APIs of OnesaitPlatform
    """
    api_manager_path = config.API_MANAGER_PATH
    api_caller_path = config.API_CALLER_PATH

    __find_template = Template("$protocol://$host$path/apis?identificacion=$identification&estado=$state&usuario=$user")
    __create_template = Template("$protocol://$host$path/apis")
    __delete_template = Template("$protocol://$host$path/apis/$identification/$version")
    __call_template = Template("$protocol://$host$path/$url")
    __list_template = Template("$protocol://$host$path//apis/user/$user")
    __request_template = Template("$protocol://$host$path$api_path")
    
    __IS_LIST_QUERY_STR = "/apis/user"
    __IS_FIND_QUERY_STR = "/apis?"
    
    NOT_SETTED_TOKEN_MSG = "Note token setted. Please use setToken(<token>) before"
    NOT_POSSIBLE_CONNECT_MSG = "Not possible to connect"

    def __init__(self, host, port=None):
        """
        Class ApiManagerClient to connect with Api-Manager and APIs of OnesaitPlatform

        @param host               Onesaitplatform host
        """
        #super().__init__(host, port=port) # only python > 3
        Client.__init__(self, host, port=port) # python > 2.7 & <= 3.7.1
        self.token = None
        log.info("Connection Params: "  + self.host + "/(" + self.api_manager_path + " - " + self.api_caller_path + ")")
        self.add_to_debug_trace("Connection Params: "  + self.host + "/(" + self.api_manager_path + " - " + self.api_caller_path + ")")
    
    def setToken(self, token):
        """ Set token (== object.token = <token>)"""
        self.token = token
    
    @property
    def __headers(self):
        return {
            RestHeaders.X_OP_APIKey.value: self.token,
            RestHeaders.ACCEPT_STR.value: RestHeaders.APP_JSON.value,
            RestHeaders.CONT_TYPE.value: RestHeaders.APP_JSON.value,
            RestHeaders.USER_AGENT.value: self.user_agent
        }
    
    def __str__(self):
        """
        String to print object info
        """
        hide_attributes = ["debug_trace"]
        info = "{}.{}".format(Client.__name__, self.__class__.__name__)
        info += "("
        for k, v in self.__dict__.items():
            if k not in hide_attributes:
                info += "{}={}, ".format(k, v)
        info = info[:-2] + ")"
        
        return info

    def to_json(self, as_string=False):
        """
        Export object to json

        @param as_string    If json dumped (String)

        @return json_obj    json-dict/ json string
        """
        json_obj = dict()
        json_obj["host"] = self.host
        json_obj["port"] = self.port
        json_obj["protocol"] = self.protocol
        json_obj["is_connected"] = self.is_connected
        json_obj["token"] = self.token        
        json_obj["proxies"] = self.proxies
        json_obj["timeout"] = self.timeout
        json_obj["avoid_ssl_certificate"] = self.avoid_ssl_certificate
        json_obj["raise_exceptions"] = self.raise_exceptions

        if as_string:
            json_obj = json.dumps(json_obj)

        log.info("Exported json {}".format(json_obj))
        self.add_to_debug_trace("Exported json {}".format(json_obj))
        return json_obj

    @staticmethod
    def from_json(json_object):
        """
        Creates a object from json-dict/ json-string

        @param json_object    json.dict/ json-string

        @return client        client object
        """
        client = None
        try:
            if type(json_object) == str:
                json_object = json.loads(json_object)
            
            json_object_keys = list(json_object.keys())
            client = ApiManagerClient(host=json_object['host'])
            if "port" in json_object_keys:
                client.port = json_object['port']
            if "is_connected" in json_object_keys:
                client.is_connected = json_object['is_connected'] 
            if "token" in json_object_keys:
                client.token = json_object['token']
            if "protocol" in json_object_keys:
                client.protocol = json_object['protocol']
            if "proxies" in json_object_keys:
                client.proxies = json_object['proxies']
            if "timeout" in json_object_keys:
                client.timeout = json_object['timeout']
            if "avoid_ssl_certificate" in json_object_keys:
                client.avoid_ssl_certificate = json_object['avoid_ssl_certificate']
            if "raise_exceptions" in json_object_keys:
                client.raise_exceptions = json_object['raise_exceptions']

            log.info("Imported json {}".format(json_object))
            client.add_to_debug_trace("Imported json {}".format(json_object))

        except Exception as e:
            log.error("Not possible to import object from json: {}".format(e))
        
        return client

    def raise_exception_if_not_token(self):
        if self.token is None:
            raise Exception(self.NOT_SETTED_TOKEN_MSG)

    def find(self, identification, state, user):
        """
        Find API rest information

        @param identification     API identification (name)
        @param state              API state
        @param user               API creation user

        @return ok, info  
        """
        _ok = False
        _res = None

        try:
            log.info("Making find: identification:{}, state:{}, user:{}".format(identification, state, user))
            self.raise_exception_if_not_token()
            url = self.__find_template.substitute(protocol=self.protocol, host=self.hostport, 
                                                path=self.api_manager_path, 
                                                identification=identification,
                                                state=state, 
                                                user=user
                                                )
            headers = self.__headers
            response = self.call(RestMethods.GET.value, url, headers=headers)
            log.info("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

            if response.status_code == 200:
                _res = response.json()
                log.info("Query result: {text}".format(text=response.text))
                self.add_to_debug_trace("Query result: {text}".format(text=response.text))
                _ok = True

            else:
                raise Exception("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to query api-manager: {exception}".format(exception=e))
            self.add_to_debug_trace("Not possible to query api-manager: {exception}".format(exception=e))
            _res = e

        return _ok, _res

    def create(self, json_obj):
        """
        Create an API rest from json object

        @param json_obj     json object of the API

        @return ok, info  
        """
        _ok = False
        _res = None

        try:
            log.info("Making create")
            if isinstance(json_obj, str):
                json_obj = json.loads(json_obj)

            self.raise_exception_if_not_token()
            url = self.__create_template.substitute(protocol=self.protocol, host=self.hostport, 
                                                path=self.api_manager_path
                                                )
            headers = self.__headers
            response = self.call(RestMethods.POST.value, url, headers=headers, body=json_obj)
            log.info("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

            if response.status_code == 200:
                _res = response.json()
                log.info("Query result: {text}".format(text=response.text))
                self.add_to_debug_trace("Query result: {text}".format(text=response.text))
                _ok = True

            else:
                raise Exception("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to query api-manager: {exception}".format(exception=e))
            self.add_to_debug_trace("Not possible to query api-manager: {exception}".format(exception=e))
            _res = e

        return _ok, _res
    
    def delete(self, identification, version):
        """
        Delete an API rest

        @param identification     API identification (name)
        @param version            API version

        @return ok, info  
        """
        _ok = False
        _res = None

        try:
            log.info("Making delete")
            if isinstance(version, str):
                version = version.replace("v", "") # case vX -> X

            self.raise_exception_if_not_token()
            url = self.__delete_template.substitute(protocol=self.protocol, host=self.hostport, 
                                                path=self.api_manager_path,
                                                identification=identification,
                                                version=version
                                                )
            headers = self.__headers
            response = self.call(RestMethods.DELETE.value, url, headers=headers)
            log.info("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

            if response.status_code == 200:
                _res = response.json()
                log.info("Query result: {text}".format(text=response.text))
                self.add_to_debug_trace("Query result: {text}".format(text=response.text))
                _ok = True

            else:
                raise Exception("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to query api-manager: {exception}".format(exception=e))
            self.add_to_debug_trace("Not possible to query api-manager: {exception}".format(exception=e))
            _res = e

        return _ok, _res

    def generate_url_request(self, name, version, path_params, query_params):
        url = ""
    	# version
        if version.startswith("/"):
    	    url = version
            
        elif version.lower().startswith("v"):
            url = "/" + version

        else:
            url = "/v" + version

        if name.startswith("/"):
            url += name
    	
        else:
            url = url + "/" + name
    		
    	# path params
        if path_params is not None:
            for param in path_params:
                url = url + "/" + param
	    	
    	# query params
        if query_params is not None:
            url += "?"
            for key, value in query_params.items():
                url = url + "&" + key + "=" + value

        return url

    def request(self, method, url=None, name=None, version=None, 
                path_params=None, query_params=None, body=None):
        """
        Make a request to an API rest

        @param method          method (hhtp method)
        @param url             url with path params and query params
        @param name            API rest name *
        @param version         API rest version *
        @param path_params     path_params *
        @param query_params    query_params *
        @param body            body

        * If url != None: api_path = generate_url_request(name, version, path_params, query_params)
        * else: api_path = url 

        @return ok, info  
        """
        _ok = False
        _res = None

        try:
            log.info("Making request: url:{}".format(url))
            self.raise_exception_if_not_token()

            if isinstance(version, int) or isinstance(version, float):
                version = str(int(version))
            
            if url is None:
                api_path = self.generate_url_request(name=name, version=version, 
                                                     path_params=path_params, query_params=query_params)
            else:
                api_path = url
            
            url = self.__request_template.substitute(protocol=self.protocol, host=self.hostport, 
                                                path=self.api_caller_path,
                                                api_path=api_path
                                                )
            headers = self.__headers
            response = self.call(method, url, headers=headers, body=body)
            log.info("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

            if response.status_code == 200:
                _res = response.json()
                log.info("Query result: {text}".format(text=response.text))
                self.add_to_debug_trace("Query result: {text}".format(text=response.text))
                _ok = True

            else:
                raise Exception("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to query api-manager: {exception}".format(exception=e))
            self.add_to_debug_trace("Not possible to query api-manager: {exception}".format(exception=e))
            _res = e

        return _ok, _res
    
    def list(self, user):
        """
        List APIs rest from user

        @param user     user

        @return ok, info  
        """
        _ok = False
        _res = None

        try:
            log.info("Making list: user: {}".format(user))
            self.raise_exception_if_not_token()
            url = self.__list_template.substitute(protocol=self.protocol, host=self.hostport, 
                                                path=self.api_manager_path,
                                                user=user
                                                )
            headers = self.__headers
            response = self.call(RestMethods.GET.value, url, headers=headers)
            log.info("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {} - {}".format(response.status_code, response.text))

            if response.status_code == 200:
                _res = response.json()
                log.info("Query result: {}".format(response.text))
                self.add_to_debug_trace("Query result: {}".format(response.text))
                _ok = True

            else:
                raise Exception("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to query api-manager: {exception}".format(exception=e))
            self.add_to_debug_trace("Not possible to query api-manager: {exception}".format(exception=e))
            _res = e

        return _ok, _res

    def __is_list_query(self, url):
        return self.__IS_LIST_QUERY_STR in url
    
    def __is_find_query(self, url):
        return self.__IS_FIND_QUERY_STR in url



from enum import Enum


class RestHeaders(Enum):
    X_OP_APIKey = "X-OP-APIKey"
    ACCEPT_STR = "Accept"
    ACCEPT_ALL = "*/*"
    APP_JSON = "application/json"
    CONT_TYPE  = "Content-type"
    USER_AGENT = "User-Agent"
    AUTHORIZATION = "Authorization"


import time


def wait(seconds_before=0, seconds_after=0):

    def decorator(func):

        def wrapper(*args, **kwargs):
            time.sleep(seconds_before)
            ret = func(*args, **kwargs)
            time.sleep(seconds_after)
            return ret

        return wrapper

    return decorator



import requests
from string import Template
import json
import logging
from onesaitplatform.base import Client
import onesaitplatform.common.config as config
from onesaitplatform.enum import QueryType, RestMethods, RestHeaders
from onesaitplatform.common.log import log

try:
    logging.basicConfig()
    log = logging.getLogger(__name__)
except:
    log.init_logging()

class ModelServiceClient(Client):
    
    # endpoints
    __endpoint_health = "/health"
    __endpoint_information = "/information"
    __endpoint_status = "/status"
    __endpoint_sample = "/sample"
    __endpoint_stats = "/stats"
    __endpoint_predict = "/predict"
    __endpoint_train = "/train"
    
    def __init__(self, host, model_endpoint, model_version, token=None, port=None): #, protocol="http", avoid_ssl_certificate=False):
        Client.__init__(self, host, port=port) # python > 2.7 & <= 3.7.1
        self.__model_endpoint = self.__parse_input_model_endpoint(model_endpoint)
        self.__model_version = self.__parse_input_model_version(model_version)
        self.token = token
        
    def __str__(self):
        """
        String to print object info
        """
        info = "{}(".format(self.__class__.__name__)
        mandatory = "host={host}, model_endpoint={model_endpoint}, model_version={model_version}, ".format(
            host=self.host, model_endpoint=self.model_endpoint, model_version=self.model_version
            )
        
        optionals = "port={port}, protocol={protocol}, avoid_ssl_certificate={avoid_ssl_certificate}, ".format(
            port=self.port, protocol = self.protocol, avoid_ssl_certificate = self.avoid_ssl_certificate
            )
        
        token_str = "token={}".format(self.token)
        if (isinstance(self.token, str)) and (len(self.token) > 50):
            token_str = "token={}".format(self.token[:20] + "..." + self.token[-20:])
        
        info += mandatory + optionals + token_str + ")"
        
        return info
    
    @property
    def model_endpoint(self):
        return self.__model_endpoint
    
    def __parse_input_model_endpoint(self, model_endpoint):
        if not model_endpoint.startswith("/"):
            model_endpoint = "/{}".format(model_endpoint)
        return model_endpoint
    
    @property
    def model_version(self):
        return self.__model_version
    
    def __parse_input_model_version(self, model_version):
        model_version = str(model_version)
        if model_version.startswith("v"):
            model_version = model_version[1:]
        return model_version
    
    @property
    def model_version_str(self):
        return self.model_version if self.model_version.startswith("v") else "v{}".format(self.model_version)
    
    @property
    def is_connected(self):
        is_healthly = False
        try:
            is_healthly = self.health().status_code == 200
        except Exception as e:
            log.warning("Not possible to connect with remote model: {}".format(e))
        finally:
            self.__is_connected = is_healthly
        return self.__is_connected
    
    @is_connected.setter
    def is_connected(self, is_con):
        self.__is_connected = is_con
    
    @property
    def suported_protocols(self):
        return self.__supported_protocols
    
    @property
    def model_path(self):
        return "{}/api/{}".format(self.model_endpoint, self.model_version_str)
    
    @property
    def model_url(self):
        url = self.host
        if self.protocol:
            url = "{}://{}".format(self.protocol, self.host)
        
        if self.port:
            url += ":{}".format(self.port)
        url += self.model_path
        
        return url

    def __set_token_if_not_none(self, token):
        if token:
            self.token = token
    
    @property
    def __headers(self):
        return {
            RestHeaders.AUTHORIZATION.value: self.token,
            RestHeaders.CONT_TYPE.value: RestHeaders.APP_JSON.value
        }
    
    def health(self, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_health)        
        return self.call(RestMethods.GET.value, url, headers=self.__headers)
    
    def status(self, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_status)
        return self.call(RestMethods.GET.value, url, headers=self.__headers)
    
    def information(self, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_information)
        return self.call(RestMethods.GET.value, url, headers=self.__headers)
    
    def sample(self, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_sample)
        return self.call(RestMethods.GET.value, url, headers=self.__headers)
    
    def stats(self, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_stats)
        return self.call(RestMethods.GET.value, url, headers=self.__headers)
    
    def predict(self, data, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_predict)
        """if not isinstance(data, str):
            data = json.dumps(data)"""
        #return requests.post(url, headers=self.__headers, data=data, proxies=self.proxies)
        return self.call(RestMethods.POST.value, url, headers=self.__headers, body=data)
    
    def train(self, data, token=None):
        self.__set_token_if_not_none(token)
        url = "{url}{op_endpoint}".format(url=self.model_url, op_endpoint=self.__endpoint_train)
        """if not isinstance(data, str):
            data = json.dumps(data)"""
        #return requests.post(url, headers=self.__headers, data=data, proxies=self.proxies)
        return self.call(RestMethods.POST.value, url, headers=self.__headers, body=data)
    
    def to_json(self, as_string=False):
        """
        Export object to json

        @param as_string    If json dumped (String)

        @return json_obj   json-dict/ json string
        """
        json_obj = dict()
        json_obj["host"] = self.host
        json_obj["port"] = self.port
        json_obj["protocol"] = self.protocol
        json_obj["is_connected"] = self.is_connected
        json_obj["proxies"] = self.proxies
        json_obj["timeout"] = self.timeout
        json_obj["avoid_ssl_certificate"] = self.avoid_ssl_certificate
        json_obj["raise_exceptions"] = self.raise_exceptions
        json_obj["model_endpoint"] = self.model_endpoint
        json_obj["model_version"] = self.model_version
        json_obj["token"] = self.token
        if as_string:
            json_obj = json.dumps(json_obj)

        log.info("Exported json {}".format(json_obj))
        self.add_to_debug_trace("Exported json {}".format(json_obj))
        return json_obj
    
    @staticmethod
    def from_json(json_object):
        """
        Creates a object from json-dict/ json-string

        @param json_object    json.dict/ json-string

        @return client        client object
        """
        client = None
        try:
            if type(json_object) == str:
                json_object = json.loads(json_object)
            
            json_object_keys = list(json_object.keys())

            client = Client(host=json_object['host'])
            if "port" in json_object_keys:
                client.port = json_object['port']
            if "protocol" in json_object_keys:
                client.protocol = json_object['protocol']
            if "is_connected" in json_object_keys:
                client.is_connected = json_object['is_connected'] 
            if "proxies" in json_object_keys:
                client.proxies = json_object['proxies']
            if "timeout" in json_object_keys:
                client.timeout = json_object['timeout']
            if "avoid_ssl_certificate" in json_object_keys:
                client.avoid_ssl_certificate = json_object['avoid_ssl_certificate']
            if "raise_exceptions" in json_object_keys:
                client.raise_exceptions = json_object['raise_exceptions']
            if "model_endpoint" in json_object_keys:
                client.model_endpoint = json_object['model_endpoint']
            if "model_version" in json_object_keys:
                client.model_version = json_object['model_version']
            if "token" in json_object_keys:
                client.token = json_object['token']
                
            log.info("Imported json {}".format(json_object))
            client.add_to_debug_trace("Imported json {}"
                                          .format(json_object))

        except Exception as e:
            log.error("Not possible to import object from json: {}".format(e))

        return client
    


import paho.mqtt.client as mqtt
import json
from threading import Event
from uuid import uuid4

from onesaitplatform.mqttclient.utils import Message


class Client:

    def __init__(self, host, port=1883, platform=None, instance=None, token=None, timeout=10):
        self._host = host
        self._port = port

        self._platform = platform
        self._instance = instance
        self._token = token

        self._timeout = timeout

        self._client = mqtt.Client(instance)
        self._client.on_connect = self._on_connect
        self._client.on_message = self._on_message

        self._conected_event = Event()
        self._messages = {}
        self._session_key = None

    @property
    def _channel(self):
        return "/topic/message/{}".format(self._instance)

    def connect(self):
        self._client.connect(self._host)
        self._client.loop_start()
        self._conected_event.wait()
        self.join()

    def disconnect(self):
        self.leave()
        self._session_key = None
        self._client.loop_stop()
        self._client.disconnect()

    def join(self):
        msg = {
            "@type": "SSAPBodyJoinMessage",
            "token": self._token,
            "clientPlatform": self._platform,
            "clientPlatformInstance": self._instance,
            "sessionKeyMandatory": False,
            "ontologyMandatory": False
        }
        ret = self._send_msg("JOIN", msg)
        self._session_key = ret.result['sessionKey']

    def leave(self):
        return
        msg = {
            "@type": "SSAPBodyLeaveMessage",
            "clientPlatform": self._platform,
            "clientPlatformInstance": self._instance,
            "sessionKeyMandatory": True,
            "ontologyMandatory": False
        }
        ret = self._send_msg("LEAVE", msg)
        self._session_key = ret.result['sessionKey']
        print("DESCONECTADO con sesion: {}".format(self._session_key))

    def query(self, ontology, query, query_type="NATIVE"):
        msg = {
            "@type": "SSAPBodyQueryMessage",
            "ontology": ontology,
            "query": 'db.{}.find({})'.format(ontology, query),
            "queryType": query_type,
            "resultFormat": None,
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("QUERY", msg)
        return resp

    def insert(self, ontology, data):
        msg = {
            "@type": "SSAPBodyInsertMessage",
            "ontology": ontology,
            "data": data,
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("INSERT", msg)
        return resp

    def delete(self, ontology, query):
        msg = {
            "@type": "SSAPBodyDeleteMessage",
            "ontology": ontology,
            "query": "db.{}.remove({})".format(ontology, query),
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("DELETE", msg)
        return resp

    def delete_id(self, ontology, inst_id):
        msg = {
            "@type": "SSAPBodyDeleteByIdMessage",
            "ontology": ontology,
            "id": inst_id,
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("DELETE_BY_ID", msg)
        return resp

    def update(self, ontology, query, set):
        msg = {
            "@type": "SSAPBodyUpdateMessage",
            "ontology": ontology,
            "query": "db.{}.update({}, {})".format(ontology, query, set),
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("UPDATE", msg)
        return resp

    def update_id(self, ontology, inst_id, data):
        msg = {
            "@type": "SSAPBodyUpdateByIdMessage",
            "ontology": ontology,
            "id": inst_id,
            "data": data,
            "sessionKeyMandatory": True,
            "ontologyMandatory": True
        }
        resp = self._send_msg("UPDATE_BY_ID", msg)
        return resp

    # The callback for when the client receives a CONNACK response from the server.
    def _on_connect(self, client, userdata, flags, rc):
        # Subscribing in on_connect() means that if we lose the connection and
        # reconnect then subscriptions will be renewed.
        self._client.subscribe("$SYS/#")
        self._client.subscribe(self._channel)
        self._conected_event.set()

    # The callback for when a PUBLISH message is received from the server.
    def _on_message(self, client, userdata, msg):
        message = json.loads(msg.payload)
        if message['direction'] == 'REQUEST':
            return
        if message['messageId'] is None:
            print(message)
        resp = self._messages.pop(message['messageId'])
        resp.result = message['body']

    def _send_msg(self, type, body, timeout=None):
        msg_id = uuid4().hex
        msg = {
            "sessionKey": self._session_key,
            "messageId": msg_id,
            "direction": "REQUEST",
            "messageType": type,
            "body": body
        }
        self._messages[msg_id] = Message(timeout=self._timeout)

        str_msg = json.dumps(msg)
        self._client.publish(self._channel, str_msg)

        return self._messages[msg_id]

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()
        return False



import os
import sys
import unittest
relative_path = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
if relative_path not in sys.path:
    sys.path.insert(0, relative_path)

from .digitalclient import DigitalClient

class DigitalClientTest(unittest.TestCase):

    HOST = "www.onesaitplatform.online"
    IOT_CLIENT = "<client>"
    IOT_CLIENT_TOKEN = "<token>"
    PROTOCOL = "https"
    AVOID_SSL_CERTIFICATE = True
    DEBUG_MODE = False
    PROXIES = None

    def setUp(self):
        client = DigitalClient(self.HOST, iot_client=self.IOT_CLIENT, iot_client_token=self.IOT_CLIENT_TOKEN)
        client.protocol = self.PROTOCOL
        client.debug_mode = self.DEBUG_MODE
        client.proxies = self.PROXIES
        self.client = client
        self.client_json = {
            'host': self.HOST,
            'port': None,
            'protocol': self.PROTOCOL,
            'iot_client': self.IOT_CLIENT,
            'iot_client_token': self.IOT_CLIENT_TOKEN,
            'is_connected': False,
            'session_key': None,
            'proxies': self.PROXIES,
            'timeout': 10000,
            'avoid_ssl_certificate': self.AVOID_SSL_CERTIFICATE,
            'raise_exceptions': False
        }

    def test_init(self):
        self.assertEqual(self.client.host, self.HOST)
        self.assertEqual(self.client.iot_client, self.IOT_CLIENT)
        self.assertEqual(self.client.iot_client_token, self.IOT_CLIENT_TOKEN)
        self.assertEqual(self.client.debug_mode, self.DEBUG_MODE)
        self.assertEqual(self.client.protocol, self.PROTOCOL)
        self.assertEqual(self.client.proxies, self.PROXIES)

    def test_to_json(self):
        self.assertEqual(self.client.to_json().keys(), self.client_json.keys())

    def test_from_json(self):
        tmp_client = DigitalClient.from_json(self.client.to_json())
        self.assertDictEqual(tmp_client.to_json(), self.client.to_json())


if __name__ == '__main__':
    unittest.main()



import time
import requests
from string import Template
import json
import logging
from onesaitplatform.base import Client
from onesaitplatform.common.utils import wait
import onesaitplatform.common.config as config
from onesaitplatform.enum import RestMethods, QueryType, RestHeaders
from onesaitplatform.common.log import log

try:
    logging.basicConfig()
    log = logging.getLogger(__name__)
except:
    log.init_logging()

TAGS_PARAM_VALUE = '{"source": "PYTHONCLIENT"}'

class DigitalClient(Client):
    """
    Class DigitalClient to connect with Iot-Broker of OnesaitPlatform
    """
    __iot_broker_path = config.IOT_BROKER_PATH
    batch_size = config.BATCH_QUERY_SIZE

    __join_template = Template("$protocol://$host$path/rest/client/join")
    __leave_template = Template("$protocol://$host$path/rest/client/leave")
    __query_template = Template("$protocol://$host$path/rest/ontology/$ontology")
    __insert_template = Template("$protocol://$host$path/rest/ontology/$ontology")
    __update_template = Template("$protocol://$host$path/rest/ontology/$ontology/update")
    __delete_template = Template("$protocol://$host$path/rest/ontology/$ontology/$entity")
    __query_batch_file_template = Template("$protocol://$host$path/rest/ontology/$ontology/file")
    __query_file_status_template = Template("$protocol://$host$path/rest/ontology/file/$queryId/status")

    __query_batch_sql = Template("$query offset $start limit $end")
    __query_batch_mongo = Template("$query.skip($start).limit($end)")

    def __init__(self, host, port=None,
                 iot_client=config.IOT_CLIENT,
                 iot_client_token=config.IOT_CLIENT_TOKEN):
        """
        Class IotBrokerClient to connect with Iot-Broker of OnesaitPlatform

        @param host               Onesaitplatform host
        @param iot_client         Onesaitplatform Iot-Client
        @param iot_client_token   Onesaitplatform iot-Client-Token
        """
        #super().__init__(host, port=port) # only python > 3
        Client.__init__(self, host, port=port) # python > 2.7 & <= 3.7.1
        self.iot_client = iot_client
        self.iot_clientId = iot_client + ":PythonClient"
        self.iot_client_token = iot_client_token
        self.session_key = None

        log.info("Created client with iot-broker \
                 host:{}, path:{}, client:{}, token:{}"
                 .format(host, self.__iot_broker_path,
                         iot_client, iot_client_token))
        self.add_to_debug_trace("Created client with \
                                iot-broker host:{}, path:{}, \
                                client:{}, token:{}"
                                .format(host, self.__iot_broker_path,
                                        iot_client, iot_client_token))

    def __str__(self):
        """
        String to print object info
        """
        hide_attributes = ["debug_trace"]
        info = "{}".format(self.__class__.__name__)
        info += "("
        for k, v in self.__dict__.items():
            if k not in hide_attributes:
                info += "{}={}, ".format(k, v)
        info = info[:-2] + ")"

        return info

    def to_json(self, as_string=False):
        """
        Export object to json

        @param as_string    If json dumped (String)

        @return json_obj    json-dict/ json string
        """
        json_obj = dict()
        json_obj["host"] = self.host
        json_obj["port"] = self.port
        json_obj["protocol"] = self.protocol
        json_obj["iot_client"] = self.iot_client
        json_obj["iot_client_token"] = self.iot_client_token
        json_obj["is_connected"] = self.is_connected
        json_obj["session_key"] = self.session_key        
        json_obj["proxies"] = self.proxies
        json_obj["timeout"] = self.timeout
        json_obj["avoid_ssl_certificate"] = self.avoid_ssl_certificate
        json_obj["raise_exceptions"] = self.raise_exceptions

        if as_string:
            json_obj = json.dumps(json_obj)

        log.info("Exported json {}".format(json_obj))
        self.add_to_debug_trace("Exported json {}".format(json_obj))
        return json_obj

    @staticmethod
    def from_json(json_object):
        """
        Creates a object from json-dict/ json-string

        @param json_object    json.dict/ json-string

        @return client        client object
        """
        client = None
        try:
            if type(json_object) == str:
                json_object = json.loads(json_object)

            json_object_keys = list(json_object.keys())
            client = DigitalClient(host=json_object['host'])
            if "port" in json_object_keys:
                client.port = json_object['port']
            if "iot_client" in json_object_keys:
                client.iot_client = json_object['iot_client']
            if "iot_client_token" in json_object_keys:
                client.iot_client_token = json_object['iot_client_token'] 
            if "is_connected" in json_object_keys:
                client.is_connected = json_object['is_connected'] 
            if "session_key" in json_object_keys:
                client.session_key = json_object['session_key']
            if "protocol" in json_object_keys:
                client.protocol = json_object['protocol']
            if "proxies" in json_object_keys:
                client.proxies = json_object['proxies']
            if "timeout" in json_object_keys:
                client.timeout = json_object['timeout']
            if "avoid_ssl_certificate" in json_object_keys:
                client.avoid_ssl_certificate = json_object['avoid_ssl_certificate']
            if "raise_exceptions" in json_object_keys:
                client.raise_exceptions = json_object['raise_exceptions']

            log.info("Imported json {}".format(json_object))
            client.add_to_debug_trace("Imported json {}".format(json_object))

        except Exception as e:
            log.error("Not possible to import object from json: {}".format(e))

        return client

    def raw_join(self, iot_client=None, iot_client_token=None):
        """
        Login in the platform with Iot-Client credentials

        @return response
        """

        if iot_client is not None:
            self.iot_client = iot_client
        if iot_client_token is not None:
            self.iot_client_token = iot_client_token

        log.info("Created connection with iot-broker host:{}, path:{}, client:{}, token:{}"
                     .format(self.host, self.__iot_broker_path, self.iot_client, self.iot_client_token))

        url = self.__join_template.substitute(protocol=self.protocol, host=self.hostport, path=self.__iot_broker_path)
        querystring = {
            "token": self.iot_client_token,
            "clientPlatform": self.iot_client,
            "clientPlatformId": self.iot_clientId,
            "tags": TAGS_PARAM_VALUE
            }
        headers = {
            RestHeaders.ACCEPT_STR.value: RestHeaders.APP_JSON.value,
            RestHeaders.CONT_TYPE.value: RestHeaders.APP_JSON.value
            }
        response = self.call(RestMethods.GET.value, url, headers, querystring)

        if response.status_code == 200:
            self.session_key = response.json()["sessionKey"]
            self.is_connected = True
            log.info("Logged correctly with session_key: {}".format(response.text))
            self.add_to_debug_trace("Logged correctly with session_key: {}".format(response.text))

        else:
            log.info("Not possible to loggin: {} - {}".format(response.status_code, response.text))
            self.add_to_debug_trace("Not possible to loggin: {} - {}".format(response.status_code, response.text))

        return response

    def join(self, iot_client=None, iot_client_token=None):
        """
        Login in the platform with Iot-Client credentials

        @return ok, info
        """
        _ok = False
        _res = None

        try:
            response = self.raw_join(iot_client=iot_client, iot_client_token=iot_client_token)

            if response.status_code == 200:
                _res = response.json()
                _ok = True

            else:
                _res = response.text

        except Exception as e:
            self.is_connected = False
            log.error("Not possible to conect with iot-broker: {}".format(e))
            self.raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def raw_leave(self):
        """
        Logout in the platform with session token

        @return response
        """
        log.info("Leaving connection with session_key:{}".format(self.session_key))
        if self.is_connected:
            url = self.__leave_template.substitute(protocol=self.protocol, host=self.hostport, path=self.__iot_broker_path)
            headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
            params = {"tags": TAGS_PARAM_VALUE}
            response = self.call(RestMethods.GET.value, url, headers=headers, params=params)
            self.is_connected = False
            self.session_key = None
            log.info("Disconnected correctly: {}".format(response.text))
            self.add_to_debug_trace("Disconnected correctly: {}".format(response.text))
        else:
            log.info("There is not connection, please join() before leave()")
            self.add_to_debug_trace("There is not connection, please join() before leave()")

        return response

    def leave(self):
        """
        Logout in the platform with session token

        @return ok, info
        """
        _ok = False
        _res = None
        try:
            if self.is_connected:
                response = self.raw_leave()
                if response.status_code == 200:
                    _res = response.text
                    _ok = True
            else:
                _ok = True

        except Exception as e:
            log.error("Not possible to disconnect with iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to disconnect with iot-broker: {}".format(e))
            self.raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def restart(self):
        """
        Restar conection:
            - leave()
            - join()

        @return ok, info
        """
        log.info("Restarting connection with session_key:{}, connected:{}".format(self.session_key, self.is_connected))
        _ok_leave, _res_leave = self.leave()
        _ok_join, _res_join = self.join()
        if not _ok_join:
            log.warning("Not possible to restart connexion with iot-broker")
            self.add_to_debug_trace("Not possible to restart connexion with iot-broker")

        return _ok_join, _res_join

    def raw_query(self, ontology, query, query_type):
        """
        Make a query to iot-broker service of the platform

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE', 'SQL']

        @return  response
        """
        assert ontology != None, "Invalid input ontology"
        assert query != None, "Invalid input query"
        assert query_type != None, "Invalid input query_type"

        log.info("Making query to ontology:{}, query:{}, query_type:{}".format(ontology, query, query_type))
        url = self.__query_template.substitute(protocol=self.protocol, host=self.hostport,
                                                path=self.__iot_broker_path, ontology=ontology)
        querystring = {"query": query, "queryType": query_type.upper(), "tags": TAGS_PARAM_VALUE}
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
        response = self.call(RestMethods.GET.value, url, headers=headers, params=querystring)

        if response.status_code != 200:
            log.warn("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            self.add_to_debug_trace("Response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
            log.info("Not possible to connect ({}) - {}, reconnecting...".format(response.status_code, response.text))
            _ok_reconnect, _res_reconnect = self.restart()
            log.info("Reconnected: {} - {}".format(_ok_reconnect, _res_reconnect))
            self.add_to_debug_trace("Reconnected: {} - {}".format(_ok_reconnect, _res_reconnect))

            if _ok_reconnect:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                response = self.call(RestMethods.GET.value, url, headers=headers, params=querystring)

        return response

    def query(self, ontology, query, query_type):
        """
        Make a query to iot-broker service of the platform

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE', 'SQL']

        @return ok, info
        """
        _ok = False
        _res = None
        try:
            response = self.raw_query(ontology, query, query_type)

            if self.is_correct_status_code(response.status_code):
                _res = response.json()
                log.info("Response: {} - {}".format(response.status_code, _res[0] if len(_res) > 0 else []))
                self.add_to_debug_trace("Response: {} - {}".format(response.status_code, _res[0] if len(_res) > 0 else []))
                _ok = True

            else:
                log.info("Bad response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                self.add_to_debug_trace("Bad response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                _res = response.text

        except Exception as e:
            log.error("Not possible to query iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to query iot-broker: {}".format(e))
            _res = e

        return _ok, _res

    def query_batch(self, ontology, query, query_type, batch_size = None, offset=None):
        """
        Make a query to iot-broker service of the platform paginated by batch size

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE', 'SQL']
        @param batch_size   batch size (default from configuration)

        @return ok, info
        """
        log.info("Making query batch to ontology:{}, query:{}, query_type:{}".format(ontology, query, query_type))

        _ok = False
        _res = None

        if batch_size is None:
            batch_size = self.batch_size

        if offset is None:
            offset = 0
        limit = batch_size

        res_query_count = batch_size
        _res = []
        while res_query_count == batch_size:

            res_query_count = 0
            step_query = self._query_batch_str(query, offset, limit, query_type)
            ok_query, res_query = None, None

            for idx in range(0,3):
                ok_query, res_query = self.query(ontology, step_query, query_type)
                if ok_query: # No retries if query is successful
                    break
                log.error("Retrying to query to iot-broker {}".format(str(idx + 1)))
                self.restart()
                time.sleep(20)


            if ok_query:
                res_query_count = len(res_query)
                _res += res_query
                offset += batch_size

            _ok = ok_query

        return _ok, _res

    def raw_query_file(self, ontology, query, query_type, responseType):
        """
        Make a query batch to iot-broker service of the platform and create a file with data returned

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE', 'SQL']
        @param responseType response type ['DISK', 'S3_MINIO', 'URL']

        @return response

        """
        assert ontology != None, "Invalid input ontology"
        assert query != None, "Invalid input query"
        assert query_type != None, "Invalid input query_type"
        assert responseType != None, "Invalid input responseType"


        log.info("Making query and file to ontology:{}, query:{}, query_type:{}, responseType:{}".format(
            ontology, query, query_type, responseType))    

        url = self.__query_batch_file_template.substitute(protocol=self.protocol, host=self.hostport,
                                               path=self.__iot_broker_path, ontology=ontology)
        querystring = {"query": query, "queryType": query_type.upper(), "responseType":responseType.upper(), "tags": TAGS_PARAM_VALUE}
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
        response = self.call(RestMethods.GET.value, url, headers=headers, params=querystring)

        if response.status_code != 200:
            log.info("Session expired, reconnecting...")
            is_reconnected, _ = self.restart()
            log.info("Reconnected: {}".format(is_reconnected))
            self.add_to_debug_trace("Reconnected: {}".format(is_reconnected))

            if is_reconnected:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                response = self.call(RestMethods.POST.value, url, headers=headers, body=body, params=params)

        return response

    def query_file(self, ontology, query, query_type, responseType):
        """
        Make a query batch to iot-broker service of the platform and create a file with data returned

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE', 'SQL']
        @param responseType response type ['DISK', 'S3_MINIO', 'URL']

        @return ok, info

        """

        _ok = False
        _res = None

        try:
            response = self.raw_query_file(ontology, query, query_type, responseType)

            if self.is_correct_status_code(response.status_code):
                _res = response.json()
                log.info("Response: {} - {}".format(response.status_code, _res))
                self.add_to_debug_trace("Response: {} - {}".format(response.status_code, _res))
                _ok = True

            else:
                log.info("Bad response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                self.add_to_debug_trace("Bad response: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                _res = response.text

        except Exception as e:
            log.error("Not possible to query iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to query iot-broker: {}".format(e))
            _res = e

        return _ok, _res

    def raw_query_file_status(self, queryId):
        """
        Make a query to now the status of the query

        @param queryId     query identifier

        @return response

        """

        assert queryId != None, "Invalid input ontology"

        log.info("Making query to know the status of the query with queryId:{}".format(queryId))

        url = self.__query_file_status_template.substitute(protocol=self.protocol, host=self.hostport,
                                               path=self.__iot_broker_path, queryId=queryId)
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
        response = self.call(RestMethods.GET.value, url, headers=headers)

        if response.status_code != 200:
            log.info("Session expired, reconnecting...")
            is_reconnected, _ = self.restart()
            log.info("Reconnected: {}".format(is_reconnected))
            self.add_to_debug_trace("Reconnected: {}".format(is_reconnected))

            if is_reconnected:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                response = self.call(RestMethods.POST.value, url, headers=headers, body=body, params=params)

        return response

    def query_file_status(self, queryId, timeout = 60, timesleep = 5):
        """
        Make a query to now the status of the query

        @param queryId     query identifier

        @return info

        """

        _res = None

        start_time = time.time()        

        try:
            while time.time() - start_time < timeout:
                
                response = self.raw_query_file_status(queryId)

                if self.is_correct_status_code(response.status_code):
                    _res = response.json()
                    log.info("Respuesta: {} - {}".format(response.status_code, _res))
                    self.add_to_debug_trace("Respuesta: {} - {}".format(response.status_code, _res))

                    if _res == 'FINISHED':
                        break
                    elif _res == 'IN_PROGRESS':
                        log.info("La consulta está en progreso, esperando 15 segundos...")
                        print("La consulta está en progreso, esperando 15 segundos...")
                        self.add_to_debug_trace("La consulta está en progreso, esperando 15 segundos...")
                        time.sleep(timesleep)
                else:
                    log.info("Respuesta incorrecta: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                    self.add_to_debug_trace("Respuesta incorrecta: {status_code} - {text}".format(status_code=response.status_code, text=response.text))
                    _res = response.text

            
            if time.time() - start_time >= timeout:
                log.info(f"Se alcanzó el tiempo de espera de {timeout} segundos.")
                self.add_to_debug_trace(f"Se alcanzó el tiempo de espera de {timeout} segundos.")
                print(f"Se alcanzó el tiempo de espera de {timeout} segundos.")

        except Exception as e:
            log.error("Not possible to query iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to query iot-broker: {}".format(e))
            _res = e

        return _res

    def _query_batch_str(self, query, offset, limit, query_type):
        step_query = None

        if query_type == QueryType.SQL.value:
            step_query = query + " offset {} limit {}".format(offset, limit)

        elif query_type == QueryType.NATIVE.value:
            step_query = query + ".skip({}).limit({})".format(offset, limit)

        return step_query

    def raw_insert(self, ontology, list_data):
        """
        Make a insert to iot-broker service of the platform

        @param ontology     ontology name
        @param list_data     list with data to insert

        @return response
        """
        assert ontology != None, "Invalid input ontology"
        assert list_data != None, "Invalid input list_data"

        if isinstance(list_data, str):
            list_data = json.loads(list_data)

        log.info("Making insert to ontology:{}, elements:{}".format(ontology, len(list_data)))

        url = self.__insert_template.substitute(protocol=self.protocol, host=self.hostport,
                                            path=self.__iot_broker_path, ontology=ontology)
        body = list_data
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
        params = {"tags": TAGS_PARAM_VALUE}
        response = self.call(RestMethods.POST.value, url, headers=headers, body=body, params=params)

        if response.status_code != 200:
            log.info("Session expired, reconnecting...")
            is_reconnected, _ = self.restart()
            log.info("Reconnected: {}".format(is_reconnected))
            self.add_to_debug_trace("Reconnected: {}".format(is_reconnected))

            if is_reconnected:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                response = self.call(RestMethods.POST.value, url, headers=headers, body=body, params=params)

        return response

    def insert(self, ontology, list_data):
        """
        Make a insert to iot-broker service of the platform

        @param ontology     ontology name
        @param list_data     list with data to insert

        @return ok, info
        """
        _ok = False
        _res = None
        try:
            response = self.raw_insert(ontology, list_data)

            if self.is_correct_status_code(response.status_code):
                _res = response.json()
                log.info("Query result: {}".format(response.text))
                self.add_to_debug_trace("Query result: {}".format(response.text))
                _ok = True

            else:
                log.info("Bad response: {} - {}".format(response.status_code, response.text))
                self.add_to_debug_trace("Bad response: {} - {}".format(response.status_code, response.text))
                _res = response.text

        except Exception as e:
            log.error("Not possible to insert with iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to insert with iot-broker: {}".format(e))
            self.raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def raw_update(self, ontology, query=None, query_type="NATIVE", data=None, where=None, return_ids=True):
        """
        Make a update to iot-broker service of the platform

        @param ontology     ontology name
        @param query        query expression
        @param query_type   query type ['NATIVE']
        @param data         Optional - data to update, only if not using query
        @param where        OptionaL - selection criteria for the update, only if not using query
        @param return_ids   Optional - return ids in response (default: True)

        @return response
        """
        assert ontology is not None, "Invalid input ontology"

        if return_ids: return_ids = "true"
        else: return_ids = "false"

        if query is None:
            assert data is not None, "Invalid data"
            assert isinstance(data, dict), "Invalid data type"
            if where is None:
                where = {}
            log.info("Making update to ontology:{}, elements:{}".format(ontology, data))

        if data is None:
            assert query is not None, "Invalid query"
            log.info("Making query update to ontology:{}, query:{}, query_type:{}".format(ontology, query, query_type))

        url = self.__update_template.substitute(protocol=self.protocol, host=self.hostport,
                                            path=self.__iot_broker_path, ontology=ontology)

        body = query

        if not query:
            parameters = {"multi": "true"}  # It could be added as a parameter on a future release
            json_data = json.dumps(data)
            body = "db.{}.update({},{},{})".format(ontology, where, json_data, parameters)

        params = {"ids": return_ids, "tags": TAGS_PARAM_VALUE}
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}

        response = self.call(RestMethods.PUT.value, url, headers=headers, params=params, body=body)

        if response.status_code != 200:
            log.info("Session expired, reconnecting...")
            is_reconnected, _ = self.restart()
            log.info("Reconnected: {}".format(is_reconnected))
            self.add_to_debug_trace("Reconnected: {}".format(is_reconnected))

            if is_reconnected:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                params = {"ids": True, "tags": TAGS_PARAM_VALUE}
                response = self.call(RestMethods.PUT.value, url, headers=headers, body=body, params=params)

        return response

    def update(self, ontology, query=None, query_type="NATIVE", data=None, where=None, return_ids=True):
        """
        Make a update to iot-broker service of the platform

        @param ontology     ontology name
        @param query        query expression
        @param query_type   quert type ['NATIVE']
        @param data         data to update
        @param where        selection criteria for the update
        @param return_ids   return ids in response (optional, default: True)

        @return ok, info
        """
        _ok = False
        _res = None
        try:
            response = self.raw_update(ontology, query, query_type, data, where, return_ids)

            if self.is_correct_status_code(response.status_code):
                _res = response.json()
                log.info("Query result: {}".format(response.text))
                self.add_to_debug_trace("Query result: {}".format(response.text))
                _ok = True

            else:
                log.info("Bad response: {} - {}".format(response.status_code, response.text))
                self.add_to_debug_trace("Bad response: {} - {}".format(response.status_code, response.text))
                _res = response.text

        except Exception as e:
            log.error("Not possible to update with iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to update with iot-broker: {}".format(e))
            self.raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def raw_delete(self, ontology, entity_id, return_ids=True):
        """
        Make a delete to iot-broker service of the platform

        @param ontology     ontology name
        @param entity_id    entity object id
        @param return_ids   return ids in response (optional, default: True)

        @return response
        """
        assert ontology != None, "Invalid input ontology"
        assert entity_id != None, "Invalid input entity_id"

        if return_ids: return_ids = "true"
        else: return_ids = "false"

        log.info("Making delete to ontology:{}, entity_id:{}".format(ontology, entity_id))

        url = self.__delete_template.substitute(protocol=self.protocol, host=self.hostport,
                                            path=self.__iot_broker_path, ontology=ontology,
                                            entity=entity_id)
        params = {"ids": return_ids, "tags": TAGS_PARAM_VALUE}
        headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
        response = self.call(RestMethods.DELETE.value, url, headers=headers, params=params)

        if response.status_code != 200:
            log.info("Session expired, reconnecting...")
            is_reconnected, _ = self.restart()
            log.info("Reconnected: {}".format(is_reconnected))
            self.add_to_debug_trace("Reconnected: {}".format(is_reconnected))

            if is_reconnected:
                headers = {RestHeaders.AUTHORIZATION.value: self.session_key}
                response = self.call(RestMethods.DELETE.value, url, headers=headers, params=params)

        return response

    def delete(self, ontology, entity_id, return_ids=True):
        """
        Make a insert to iot-broker service of the platform

        @param ontology     ontology name
        @param entity_id    entity object id
        @param return_ids   return ids in response (optional, default: True)

        @return ok, info
        """
        _ok = False
        _res = None
        try:
            response = self.raw_delete(ontology, entity_id, return_ids)

            if self.is_correct_status_code(response.status_code):
                _res = response.json()
                log.info("Query result: {}".format(response.text))
                self.add_to_debug_trace("Query result: {}".format(response.text))
                _ok = True

            else:
                log.info("Bad response: {} - {}".format(response.status_code, response.text))
                self.add_to_debug_trace("Bad response: {} - {}".format(response.status_code, response.text))
                _res = response.text

        except Exception as e:
            log.error("Not possible to delete with iot-broker: {}".format(e))
            self.add_to_debug_trace("Not possible to delete with iot-broker: {}".format(e))
            self.raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def connect(self):
        return self.join()

    def disconnect(self):
        return self.leave()

    def __enter__(self):
        self.join()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.leave()
        return False



#!/usr/bin/env python

from os import path, walk
import sys
from setuptools import setup, find_packages

NAME = "onesaitplatform-client-services"
VERSION = "1.4.12"

DESCRIPTION = "Python Implementation of the Onesait Platform utilities"
LONG_DESCRIPTION = open(path.join(path.dirname(__file__), 'README.md')).read()

LICENSE = "Apache v2.0"

KEYWORDS = (
    'onesaitplatform client services',
)

PACKAGES = find_packages()

REQUIRED_PACKAGES = ["paho-mqtt", "six", "requests"]

NAMESPACE_PACKAGES = ["onesaitplatform"]


if __name__ == '__main__':

    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        license=LICENSE,
        packages=PACKAGES,
        url="",
        install_requires=REQUIRED_PACKAGES,
        keywords=KEYWORDS,
        namespace_packages=NAMESPACE_PACKAGES,
        include_package_data=True,
        zip_safe=False,
    )



import os
import time
import json
import logging
import requests
import onesaitplatform.common.config as config
from onesaitplatform.enum import RestProtocols
from onesaitplatform.common.log import log

try:
    logging.basicConfig()
    log = logging.getLogger(__name__)
except:
    log.init_logging()


class Client:
    """
    Class Client as base class of connections
    """
    user_agent = config.USER_AGENT
    debug_trace_limit = config.DEBUG_TRACE_LIMIT
    debug_mode = False

    __supported_protocols = [RestProtocols.HTTP.value, RestProtocols.HTTPS.value]
    __avoid_ssl_certificate = False

    def __init__(self, host, port=None):
        """
        Class Client as base class of connections

        @param host               Onesaitplatform host
        """
        self.host = host
        self.port = port
        self.__timeout = None
        self.__proxies = None
        self.__raise_exceptions = False
        self.__protocol = config.PROTOCOL
        self.__avoid_ssl_certificate = False
        self.__is_connected = False
        self.debug_trace = []
        self.load_proxies_automatically()

    @property
    def hostport(self):
        hostport = self.host
        if self.port is not None:
            hostport += ":{}".format(self.port)
        return hostport

    @property
    def timeout(self):
        return self.__timeout

    @timeout.setter
    def timeout(self, timeout):
        try:
            timeout = int(timeout)
        except:
            timeout = None
        self.__timeout = timeout

    @property
    def proxies(self):
        return self.__proxies

    @proxies.setter
    def proxies(self, proxies):
        if not isinstance(proxies, dict):
            proxies = None
        self.__proxies = proxies
    
    @property
    def protocol(self):
        return self.__protocol

    @protocol.setter
    def protocol(self, protocol):
        if isinstance(protocol, RestProtocols):
            protocol = protocol.value
        
        if not isinstance(protocol, str):
            raise ValueError("Not supported protocol value type (only str or RestProtocols)")
        else:
            protocol = protocol.lower()

        if protocol == RestProtocols.HTTPS.value:
            self.__protocol = protocol
        else:    
            self.__protocol = RestProtocols.HTTP.value
            self.avoid_ssl_certificate = False

    @property
    def is_connected(self):
        return self.__is_connected
    
    @is_connected.setter
    def is_connected(self, is_con):
        self.__is_connected = bool(is_con)
    
    @property
    def avoid_ssl_certificate(self):
        return self.__avoid_ssl_certificate

    @avoid_ssl_certificate.setter
    def avoid_ssl_certificate(self, avoid_ssl_certificate):
        if self.protocol == RestProtocols.HTTPS.value:
            self.__avoid_ssl_certificate = bool(avoid_ssl_certificate)
        else:    
            self.__avoid_ssl_certificate = False

    @property
    def raise_exceptions(self):
        return self.__raise_exceptions

    @raise_exceptions.setter
    def raise_exceptions(self, raise_exceptions):
        if raise_exceptions == True or raise_exceptions == 1:
            self.__raise_exceptions = True
        else:    
            self.__raise_exceptions = False
    
    def add_to_debug_trace(self, msg):
        """
        Add a message to debug trace list
        """
        to_add = "[{}] {}".format(time.ctime(), msg)
        if self.debug_mode:
            print(to_add)
        if len(self.debug_trace) == self.debug_trace_limit:
            self.debug_trace.pop()
        self.debug_trace.append(to_add)

    def to_json(self, as_string=False):
        """
        Export object to json

        @param as_string    If json dumped (String)

        @return json_obj   json-dict/ json string
        """
        json_obj = dict()
        json_obj["host"] = self.host
        json_obj["port"] = self.port
        json_obj["protocol"] = self.protocol
        json_obj["is_connected"] = self.is_connected
        json_obj["proxies"] = self.proxies
        json_obj["timeout"] = self.timeout
        json_obj["avoid_ssl_certificate"] = self.avoid_ssl_certificate
        json_obj["raise_exceptions"] = self.raise_exceptions
        if as_string:
            json_obj = json.dumps(json_obj)

        log.info("Exported json {}".format(json_obj))
        self.add_to_debug_trace("Exported json {}".format(json_obj))
        return json_obj

    def is_correct_status_code(self, status_code):
        assert isinstance(status_code, int), "Invalid input value"
        return (status_code >= 200) and (status_code < 300)

    def load_proxies_automatically(self):
        proxies = {}
        if "http_proxy" in os.environ.keys():
            proxies["http"] = os.environ["http_proxy"]
        elif "HTTP_PROXY" in os.environ.keys():
            proxies["http"] = os.environ["HTTP_PROXY"]

        if "https_proxy" in os.environ.keys():
            proxies["https"] = os.environ["https_proxy"]
        elif "HTTPS_PROXY" in os.environ.keys():
            proxies["https"] = os.environ["HTTPS_PROXY"]

        if proxies != {}:
            log.info("Detected proxies: {}".format(proxies))
            self.add_to_debug_trace("Detected proxies: {}".format(proxies))
            self.__proxies = proxies

    @staticmethod
    def from_json(json_object):
        """
        Creates a object from json-dict/ json-string

        @param json_object    json.dict/ json-string

        @return client        client object
        """
        client = None
        try:
            if type(json_object) == str:
                json_object = json.loads(json_object)
            
            json_object_keys = list(json_object.keys())

            client = Client(host=json_object['host'])
            if "port" in json_object_keys:
                client.port = json_object['port']
            if "protocol" in json_object_keys:
                client.protocol = json_object['protocol']
            if "is_connected" in json_object_keys:
                client.is_connected = json_object['is_connected'] 
            if "proxies" in json_object_keys:
                client.proxies = json_object['proxies']
            if "timeout" in json_object_keys:
                client.timeout = json_object['timeout']
            if "avoid_ssl_certificate" in json_object_keys:
                client.avoid_ssl_certificate = json_object['avoid_ssl_certificate']
            if "raise_exceptions" in json_object_keys:
                client.raise_exceptions = json_object['raise_exceptions']

            log.info("Imported json {}".format(json_object))
            client.add_to_debug_trace("Imported json {}"
                                          .format(json_object))

        except Exception as e:
            log.error("Not possible to import object from json: {}".format(e))

        return client

    def raise_exception_if_enabled(self, exception):
        if self.raise_exceptions:
            assert isinstance(exception, Exception)
            raise exception

    def call(self, method, url, headers=None, params=None, body=None):
        """
        Make an HTTP request

        @param metod   HTTP method ['GET', 'PUT', ...]
        @param url     url path to append to host
        @param header  request headers
        @param params  request params
        @param body    request body

        @return requests.request(...) 
        """
        method = method.upper()
        log.info("Calling rest api, method:{}, url:{}, headers:{}, params:{}"
        .format(method, url, headers, params))
        self.add_to_debug_trace("Calling rest api, method:{}, url:{}, headers:{}, params:{}"
        .format(method, url, headers, params))

        response = requests.request(method, url, headers=headers, params=params, json=body, verify=not self.avoid_ssl_certificate,
                                    timeout=self.timeout, proxies=self.proxies)
        log.info("Call rest api response: {}".format(response))
        self.add_to_debug_trace("Call rest api response: {}".format(response))

        return response



import os
import requests
from string import Template
import json
import logging
import onesaitplatform.common.config as config
from onesaitplatform.enum import RestHeaders
from onesaitplatform.enum import RestMethods
from onesaitplatform.enum import RestProtocols
from onesaitplatform.common.log import log

try:
    logging.basicConfig()
    log = logging.getLogger(__name__)
except:
    log.init_logging()


class FileManager:
    """
    Class FileManager to make operations with binary repository
    """
    __path = config.FILE_MANAGER_PATH
    __binary_files_path = config.FILE_MANAGER_BINARY_FILES_PATH
    __files_path = config.FILE_MANAGER_FILES_PATH
    __get_user_info = Template("$protocol://$host$path/api/multitenant/me")
    __upload_template = Template("$protocol://$host$path")
    __update_template = Template("$protocol://$host$path/$id_file")
    __download_template = Template("$protocol://$host$path/gridfs/$id_file")
    __download_template_legacy = Template("$protocol://$host$path/$id_file")
    __download_template_minio = Template("$protocol://$host$path/api/objectstorage")
    __MAX_X_OP_APIKEY_LENGTH = 35

    __avoid_ssl_certificate = False

    def __init__(self, host, user_token=config.USER_TOKEN, legacy=False):
        """
        Class FileManager to make operations with binary repository

        @param host               Onesaitplatform host
        @param user_token         Onesaitplatform user-token
        """
        self.host = host
        self.user_token = user_token
        self.__protocol = config.PROTOCOL
        if "OSP_FILEMANAGER_DISABLE_VERIFY" in os.environ.keys():
            if os.environ["OSP_FILEMANAGER_DISABLE_VERIFY"].lower() == 'true':
                self.avoid_ssl_certificate = True
            else:
                self.avoid_ssl_certificate = False
        else:
            self.avoid_ssl_certificate = False
        self.__proxies = None
        self.__timeout = None
        self.__raise_exceptions = False
        if legacy:
            self.__download_template = self.__download_template_legacy
        self.load_proxies_automatically()
        self.json_data = None
        self.username = None
        self.tenant = None
        self.vertical = None
        self.name = None
        self.authorities = None
        self.email = None
        self.__get_user_info_multitenant()
    
    @property
    def protocol(self):
        return self.__protocol

    @protocol.setter
    def protocol(self, protocol):
        if protocol == RestProtocols.HTTPS.value:
            self.__protocol = protocol
        else:    
            self.__protocol = RestProtocols.HTTP.value
            self.__avoid_ssl_certificate = False

    @property
    def timeout(self):
        return self.__timeout

    @timeout.setter
    def timeout(self, timeout):
        try:
            timeout = int(timeout)
        except:
            timeout = None
        self.__timeout = timeout

    @property
    def proxies(self):
        return self.__proxies

    @proxies.setter
    def proxies(self, proxies):
        if not isinstance(proxies, dict):
            proxies = None
        self.__proxies = proxies
    
    @property
    def avoid_ssl_certificate(self):
        return self.__avoid_ssl_certificate

    @avoid_ssl_certificate.setter
    def avoid_ssl_certificate(self, avoid_ssl_certificate):
        if self.protocol == RestProtocols.HTTPS.value:
            self.__avoid_ssl_certificate = avoid_ssl_certificate
        else:    
            self.__avoid_ssl_certificate = False

    @property
    def raise_exceptions(self):
        return self.__raise_exceptions

    @raise_exceptions.setter
    def raise_exceptions(self, raise_exceptions):
        if raise_exceptions == True or raise_exceptions == 1:
            self.__raise_exceptions = True
        else:    
            self.__raise_exceptions = False

    @property
    def __headers(self):
        _headers =  dict()
        if len(self.user_token) < self.__MAX_X_OP_APIKEY_LENGTH:
            _headers[RestHeaders.X_OP_APIKey.value] = self.user_token
        else:
            _headers[RestHeaders.AUTHORIZATION.value] = self.user_token

        return _headers

    @property
    def __headers_download(self):
        _headers =  dict(self.__headers)
        _headers[RestHeaders.ACCEPT_STR.value] = RestHeaders.ACCEPT_ALL.value
        return _headers
    
    def __str__(self):
        """
        String to print object info
        """
        hide_attributes = []
        info = "{}".format(self.__class__.__name__)
        info += "("
        for k, v in self.__dict__.items():
            if k not in hide_attributes:
                info += "{}={}, ".format(k, v)
        info = info[:-2] + ")"

        return info

    def to_json(self, as_string=False):
        """
        Export object to json

        @param as_string    If json dumped (String)

        @return json_obj    json-dict/ json string
        """
        json_obj = dict()
        json_obj["host"] = self.host
        json_obj["protocol"] = self.protocol
        json_obj["user_token"] = self.user_token
        json_obj["proxies"] = self.proxies
        json_obj["timeout"] = self.timeout
        json_obj["avoid_ssl_certificate"] = self.avoid_ssl_certificate
        json_obj["raise_exceptions"] = self.raise_exceptions

        if as_string:
            json_obj = json.dumps(json_obj)

        log.info("Exported json {}".format(json_obj))
        return json_obj

    @staticmethod
    def from_json(json_object):
        """
        Creates a object from json-dict/ json-string

        @param json_object    json.dict/ json-string

        @return client        client object
        """
        client = None
        try:
            if type(json_object) == str:
                json_object = json.loads(json_object)

            json_object_keys = list(json_object.keys())
            client = FileManager(host=json_object['host'])
            if "user_token" in json_object_keys:
                client.user_token = json_object['user_token'] 
            if "protocol" in json_object_keys:
                client.protocol = json_object['protocol']
            if "proxies" in json_object_keys:
                client.proxies = json_object['proxies']
            if "timeout" in json_object_keys:
                client.timeout = json_object['timeout']
            if "avoid_ssl_certificate" in json_object_keys:
                client.avoid_ssl_certificate = json_object['avoid_ssl_certificate']
            if "raise_exceptions" in json_object_keys:
                client.raise_exceptions = json_object['raise_exceptions']

            log.info("Imported json {}".format(json_object))

        except Exception as e:
            log.error("Not possible to import object from json: {}".format(e))

        return client

    def __raise_exception_if_enabled(self, exception):
        if self.raise_exceptions:
            assert isinstance(exception, Exception)
            raise exception

    def load_proxies_automatically(self):
        if "OSP_FILEMANAGER_AVOID_LOADPROXIES" in os.environ.keys() and os.environ["OSP_FILEMANAGER_AVOID_LOADPROXIES"].lower() == 'true':
            self.__proxies = None
            self.proxies = None
        else:
            proxies = {}
            if "http_proxy" in os.environ.keys():
                proxies["http"] = os.environ["http_proxy"]
            elif "HTTP_PROXY" in os.environ.keys():
                proxies["http"] = os.environ["HTTP_PROXY"]

            if "https_proxy" in os.environ.keys():
                proxies["https"] = os.environ["https_proxy"]
            elif "HTTPS_PROXY" in os.environ.keys():
                proxies["https"] = os.environ["HTTPS_PROXY"]
            if proxies != {}:
                log.info("Detected proxies: {}".format(proxies))
                self.__proxies = proxies

    def __get_user_info_multitenant(self):
        """
        Get user info from onesait platform

        @param token    user token X-OP-APIKey

        @return user_info
        """
        
        _res = None
        try:
            url = self.__get_user_info.substitute(protocol=self.protocol,
                                                host=self.host,
                                                path=self.__path)
            headers = self.__headers

            response = requests.request(RestMethods.GET.value, url,
                                        headers=headers, data="",
                                        verify=not self.avoid_ssl_certificate,
                                        timeout=self.timeout, proxies=self.proxies)

            if response.status_code == 200:
                _res = json.loads(response.content)
                self.username = _res["username"]
                self.tenant = _res["tenant"]
                self.vertical = _res["vertical"]
                self.name = _res["name"]
                self.authorities = _res["authorities"]
                self.email = _res["email"]

        except Exception as e:
            log.error("Not possible to get user info: {}".format(e))
            self.__raise_exception_if_enabled(e)
  
    def upload_file(self, filename, filepath, metadata = None):
        """
        Upload a file to binary-repository

        @param filename           file name
        @param filepath           file path
        @param metadata           file metadata or description
        """
        _ok = False
        _res = None
        try:
            if not os.path.exists(filepath):
                raise IOError("Source file not found: {}".format(filepath))

            url = self.__upload_template.substitute(protocol=self.protocol,
                                                  host=self.host,
                                                  path=self.__binary_files_path)
            headers = self.__headers
            files_to_up = {'file': (
                filename,
                open(filepath, 'rb'),
                "multipart/form-data"
                )}
            params = {
                "metadata": metadata
            }

            response = requests.request(RestMethods.POST.value, url,
                                        params=params,
                                        headers=headers, files=files_to_up, 
                                        verify=not self.avoid_ssl_certificate,
                                        timeout=self.timeout, proxies=self.proxies)

            if response.status_code == 201:
                _ok = True
                _res = {"id": response.text,
                        "msg": "Succesfully uploaded file"
                        }

            else:
                raise Exception("Response: {status_code} - {text}"
                                .format(status_code=response.status_code, text=response.text))

        except Exception as e:
            log.error("Not possible to upload file: {}".format(e))
            self.__raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def update_file(self, id_file, filename, filepath, metadata = None):
        """
        Upload a file to binary-repository

        @param id_file            file id in onesait platform
        @param filename           file name
        @param filepath           file path
        @param metadata           file metadata or description
        """
        _ok = False
        _res = None
        try:
            if not os.path.exists(filepath):
                raise IOError("Source file not found: {}".format(filepath))

            url = self.__update_template.substitute(protocol=self.protocol,
                                                    host=self.host,
                                                    path=self.__binary_files_path,
                                                    id_file=id_file)
            
            headers = self.__headers
            files_to_up = {'file': (
                filename,
                open(filepath, 'rb'),
                "multipart/form-data"
                )}
            params = {
                "metadata": metadata
            }

            response = requests.request(RestMethods.POST.value, url,
                                        params=params,
                                        headers=headers, files=files_to_up, 
                                        verify=not self.avoid_ssl_certificate,
                                        timeout=self.timeout, proxies=self.proxies)

            if response.status_code == 202:
                _ok = True
                _res = {"id": response.text,
                        "msg": "Succesfully uploaded file"
                        }

            else:
                raise Exception("Response: {} - {}"
                                .format(response.status_code, response.text))

        except Exception as e:
            log.error("Not possible to upload file: {}".format(e))
            self.__raise_exception_if_enabled(e)
            _res = e

        return _ok, _res

    def download_file(self, id_file, filepath = ""):
        """
        Download a file from binary-repository

        @param id_file            file id in onesait platform
        @param filepath           file path to be saved
        """
        
        def get_name_from_response(response):
            name_getted = None
            key_name = 'Content-Disposition'
            name_getted = response.headers[key_name].replace(" ", "").split("=")[1].strip()
            return name_getted
        
        _ok = False
        _res = None
        try:
            url = self.__download_template.substitute(protocol=self.protocol,
                                                    host=self.host,
                                                    path=self.__files_path,
                                                    id_file=id_file)
            headers = self.__headers_download

            response = requests.request(RestMethods.GET.value, url,
                                        headers=headers, data="",
                                        verify=not self.avoid_ssl_certificate,
                                        timeout=self.timeout, proxies=self.proxies)

            if response.status_code == 200:
                _ok = True
                name_file = get_name_from_response(response)
                if os.path.isdir(filepath):
                    name_file = os.path.join(filepath, name_file)
                _res = {"id": id_file,
                        "msg": "Succesfully downloaded file",
                        "name": name_file
                        }

                with open(name_file, 'wb') as f:
                    f.write(response.content)

            else:
                raise Exception("Response: {} - {}"
                                .format(response.status_code, response.text))

        except Exception as e:
            log.error("Not possible to download file: {}".format(e))
            self.__raise_exception_if_enabled(e)
            _res = e
        
        return _ok, _res

    def download_file_minio(self, filepath):
        """
        Download a file from the minio storage

        @param filepath           file path in onesait palatform

        @return ok, file data              

        """
        

        assert filepath != None, "Invalid input filepath"

        _ok = False
        _res = None
        try:
            url = self.__download_template_minio.substitute(protocol=self.protocol,
                                                    host=self.host,
                                                    path=self.__path)
            headers = self.__headers_download
            params = {"filePath": filepath}

            response = requests.request(RestMethods.GET.value, url,
                                        headers=headers, params=params,
                                        verify=not self.avoid_ssl_certificate,
                                        timeout=self.timeout, proxies=self.proxies)

            if response.status_code == 200:
                _ok = True                
                _res = json.loads(response.content)  
                self.json_data = _res

        except Exception as e:
            log.error("Not possible to download the file: {}".format(e))
            self.add_to_debug_trace("Not possible to download the file: {}".format(e))
            _res = e

        return self

    
    def to_pandasdataframe(self, json_data=None, sep='.'):
        """
        Function to flatten a JSON and return a list of dictionaries with the flat structure.
        
        @param json_data                parsed JSON.
        @param sep                      Separator (default is '.')

        @return flattened_data          List of dictionaries with the flat structure.
        """
        
        if json_data is None:
            json_data = self.json_data 

        if not isinstance(json_data, (list, dict)):
            raise ValueError("json_data debe ser una lista o un diccionario.")

        def extract_data(d, parent_key='', sep='.'):
            items = []
            for k, v in d.items():
                new_key = f"{parent_key}{sep}{k}" if parent_key else k
                if isinstance(v, dict): 
                    items.extend(extract_data(v, new_key, sep=sep).items())
                elif isinstance(v, list): 
                    for i, sub_item in enumerate(v):
                        items.extend(extract_data(sub_item, f"{new_key}.{i}", sep=sep).items())
                else:
                    items.append((new_key, v))  
            return dict(items)

        flattened_data = [extract_data(item, sep=sep) for item in json_data]
        
        return flattened_data



import os
import sys
import unittest
relative_path = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
if relative_path not in sys.path:
    sys.path.insert(0, relative_path)

from .filemanager import FileManager

class FileManagerTest(unittest.TestCase):

    HOST = "www.onesaitplatform.online"
    USER_TOKEN = "<token>"
    PROTOCOL = "https"
    AVOID_SSL_CERTIFICATE = True
    PROXIES = None

    def setUp(self):
        client = FileManager(self.HOST, user_token=self.USER_TOKEN)
        client.protocol = self.PROTOCOL
        client.proxies = self.PROXIES
        self.client = client
        self.client_json = {
            'host': self.HOST,
            'protocol': self.PROTOCOL,
            'user_token': self.USER_TOKEN,
            'proxies': self.PROXIES,
            'timeout': 10000,
            'avoid_ssl_certificate': self.AVOID_SSL_CERTIFICATE,
            'raise_exceptions': False
        }

    def test_init(self):
        self.assertEqual(self.client.host, self.HOST)
        self.assertEqual(self.client.user_token, self.USER_TOKEN)
        self.assertEqual(self.client.protocol, self.PROTOCOL)
        self.assertEqual(self.client.proxies, self.PROXIES)

    def test_to_json(self):
        self.assertEqual(self.client.to_json().keys(), self.client_json.keys())

    def test_from_json(self):
        tmp_client = FileManager.from_json(self.client.to_json())
        self.assertDictEqual(tmp_client.to_json(), self.client.to_json())

    @unittest.skip("Developing test for update file (skipping)")
    def test_update_file(self):
        HOST = "<host>"
        PROTOCOL = "https"
        TOKEN = "<token>"
        AVOID = True
        PROXIES = {
            "http": "<http_proxy>",
            "https": "<http_proxy>"
        }
        ID_FILE = "<id_file>"
        NAME_FILE = "<name_file>"
        PATH_FILE = r"<path_file>"
        METADATA = "File updated by python with love!"
        
        manager = FileManager(HOST, TOKEN)
        manager.protocol = PROTOCOL
        manager.avoid_ssl_certificate = AVOID
        manager.proxies = PROXIES
        _ok, _res = manager.update_file(ID_FILE, NAME_FILE, PATH_FILE, METADATA)
        self.assertTrue(_ok)



if __name__ == '__main__':
    unittest.main()
